package multex; //Awt.java

//Change history:
//2001-06-28  Knabe  Any Component usable to determine the owner Window
//2000-12-12  Knabe  Methods report(...) modularized,
//                   new methods reportNonmodally(...)
//2000-10-25  Knabe  Extracted from class multex.Msg
//2000-07-17  Knabe  Auch report(Applet, Throwable)
//2000-07-11  Knabe  Button "Show Stack Trace" verschwindet nach Bettigung
//2000-06-28  Knabe  Zeilenanzahl der TextArea aus ihrem Inhalt ermitteln
//2000-06-21  Knabe  Auch Anzeige des Stack Trace im Meldungsfenster
//2000-05-23  Knabe  Erweitert um Dialog als owner
//1999-05-20  Knabe  Mit OK-Button zum Quittieren der Meldung per Return/Maus.
//1999-04-29  Knabe  Hintergrundfarbe weiss im Meldungsfenster
//1999-03-18  Knabe  erstellt in Klasse multex.Msg


/**Services for reporting onto AWT windows the messages for any exception.
  The exception can be <UL>
    <LI> with parameters (subclass of Exc)
    <LI> with cause chain and parameters (subclass of Failure)
    <LI> or any standard Java exception (other subclass of java.lang.Throwable)
  </UL>

  @author Christoph Knabe, Berlin, Copyright 1999-2001
*/
public class Awt {

//static message parts:
/**The prefix to be appended to the name of the GUI class owning the error
  message
*/
private static final String _titlePrefix = "Error Message for ";

/**The name of this class in the format package.class*/
private static final String _className = new Awt().getClass().getName();

/**Reports i_throwable using a shared owner Frame.
  As the shared owner Frame is not visible, there is no use in offering
  a modal and a non-modal version of this method.
  Better than using this version of report(), you should call
  {@link #report(java.awt.Component,Throwable)} and give it the owner Window
  or a component, from which the owner Window can be determined.
  <P>
  Reports the same contents as {@link #report(java.awt.Component,Throwable,boolean)}.
*/
public static void report(final Throwable i_throwable){
  report(getSharedOwnerFrame(), i_throwable, /*modal*/false);
}

/**Reports i_throwable blocking the owner Window.
  Reports the same contents as {@link #report(java.awt.Component,Throwable,boolean)}.
*/
public static void report(
  final java.awt.Component io_ownerHook, final Throwable i_throwable
){
  report(io_ownerHook, i_throwable, /*modal*/true);
}

/**Reports i_throwable optionally blocking the owner Window, detail description here.
  Firstly reports the same as {@link Msg#printMessages(StringBuffer, Throwable)}
  into a pop-up window of appropriate size.
  Accepting the message is done by <UL>
  <LI> clicking on the "close window" icon, or
  <LI> clicking on the OK-button, or
  <LI> typing &lt;RETURN&gt; on the OK-button </UL>
  Each of these will close the dialog box.
  <P>
  The Button "Show Stack Trace" will add the compactified
  stack trace including the chain of all causing exceptions
  to the message dialog box. The contents of the compact
  stack trace are described at method
  {@link Msg#printStackTrace(StringBuffer,Throwable)}.
  <P>
  The owner Window of the dialog box will be determined from io_ownerHook
  by searching the nearest parent Window.
  If i_modal holds, will block further input to the owner Window,
  until this dialog box will be closed.
  <P>
  In an application this method should not be called directly from
  a GUI class, but the author of each GUI class should set up a method <PRE>
  private void _report(Throwable e){Awt.report(this,e);}
  </PRE>, which will be called from each catch-clause in the event listeners
  and adds the actual GUI class as the owner of the message dialog.

  @see Msg#printReport(StringBuffer,Throwable)
*/
public static void report(
  final java.awt.Component io_ownerHook, final Throwable i_throwable,
  final boolean i_modal
){
  try{
    final java.awt.Window ownerWindow = _getOwnerWindow(io_ownerHook);
    new Dialog(ownerWindow, i_throwable, i_modal);
  }catch(final Throwable ex){_reportNoWindow(ex,i_throwable);}
  //end try
}

/**Reports i_throwable not blocking the owner Window.
  Reports the same contents as {@link #report(java.awt.Component,Throwable,boolean)}.
*/
public static void reportNonmodal(
  final java.awt.Component io_ownerHook, final Throwable i_throwable
){
  report(io_ownerHook, i_throwable, /*modal*/false);
}

/**A message reporting dialog window*/
private static class Dialog {


  /**Creates a dialog window for reporting i_throwable with owner Window io_owner
    and modality i_modal.
    If io_owner is neither a Frame nor a Dialog, the shared hidden Frame of
    multex.Awt will be used as owner window.
    This Dialog will get a title using the title of io_owner with a prefix
    indicating, that this Dialog is an error message.
    @see #report(java.awt.Frame,Throwable)
  */
  private Dialog (
    final java.awt.Window io_owner, final Throwable i_throwable,
    final boolean i_modal
  ){
    final String ownerTitle;
    if(io_owner instanceof java.awt.Frame){
      final java.awt.Frame owner = (java.awt.Frame)io_owner;
      _dialog = new java.awt.Dialog(owner);
      ownerTitle = owner.getTitle();
    }else if(io_owner instanceof java.awt.Dialog){
      final java.awt.Dialog owner = (java.awt.Dialog)io_owner;
      _dialog = new java.awt.Dialog(owner);
      ownerTitle = owner.getTitle();
    }else{
      _dialog = new java.awt.Dialog(getSharedOwnerFrame());
      ownerTitle = "";
    }
    _dialog.setTitle(_titlePrefix + ownerTitle);
    _dialog.setModal(i_modal);
    _initialize(i_throwable);
  }//Dialog

  /**Adds to _dialog the necessary components and their listeners*/
  private void _initialize(final Throwable i_throwable){

    //Look:

    //Button "OK":
    final java.awt.Button okButton
    = new java.awt.Button(" O  K ");
    //okButton.setBackground(java.awt.Color.white); //???nur fr FensterFoto
    //Button "Show Stack Trace":
    final java.awt.Button traceButton
    = new java.awt.Button("Show Stack Trace");
    //traceButton.setBackground(java.awt.Color.white); //???nur fr FensterFoto
    //buttonPanel:
    final java.awt.Panel buttonPanel
    = new java.awt.Panel(new java.awt.BorderLayout());
    buttonPanel.add(okButton);
    buttonPanel.add(traceButton, java.awt.BorderLayout.EAST);
    _dialog.add(buttonPanel, java.awt.BorderLayout.SOUTH);

    //TextArea for reporting the causer chain messages:
    // empty: 1 row, 80 columns:
    final java.awt.TextArea textArea = new java.awt.TextArea(1,80);
    final StringBuffer messageChain = new StringBuffer();
    MsgText.appendMessageChain(messageChain, i_throwable, Failure.lineSeparator);
    textArea.setBackground(java.awt.Color.white);
    textArea.setForeground(java.awt.Color.black);
    _append(textArea, messageChain.toString());
    textArea.setEditable(false);
    _dialog.add(textArea, java.awt.BorderLayout.CENTER);

    _dialog.pack();

    //Behaviour:

    okButton.addKeyListener( new java.awt.event.KeyAdapter(){
      public void keyTyped(java.awt.event.KeyEvent ev){
        if(ev.getKeyChar()=='\n'){_dispose();}
      }
    });

    okButton.addActionListener( new java.awt.event.ActionListener(){
      public void actionPerformed(final java.awt.event.ActionEvent i_){
        _dispose();
      }
    });

    traceButton.addKeyListener( new java.awt.event.KeyAdapter(){
      public void keyTyped(java.awt.event.KeyEvent ev){
        if(ev.getKeyChar()=='\n'){
          _fireTraceButton(traceButton, textArea, i_throwable);
        }
      }
    });

    traceButton.addActionListener( new java.awt.event.ActionListener(){
      public void actionPerformed(final java.awt.event.ActionEvent i_){
        _fireTraceButton(traceButton, textArea, i_throwable);
      }
    });

    _dialog.addWindowListener( new java.awt.event.WindowAdapter(){
      public void windowClosing(final java.awt.event.WindowEvent i_){
        _dispose();
      }
    });

    final java.awt.Toolkit toolkit = _dialog.getToolkit();
    toolkit.beep();
    _dialog.setVisible(true);
    //Testweise eingefgt 01-06-28:
    //_printParents(okButton);
  }//_initialize()

  /**Free the resources associated with this dialog window*/
  private void _dispose(){_dialog.dispose();}

  /**Appends the stack trace of i_throwable to io_textArea
    and disables io_traceButton
  */
  private void _fireTraceButton(final java.awt.Button io_traceButton,
    final java.awt.TextArea io_textArea, final Throwable i_throwable
  ){
    io_traceButton.setVisible(false);
    final StringBuffer buffer = new StringBuffer();
    buffer.append(Failure.lineSeparator);   buffer.append(Failure.lineSeparator);
    buffer.append(Msg.stackTraceFollows);
    buffer.append(Failure.lineSeparator);
    Msg.printStackTrace(buffer, i_throwable);
    final String stackTrace = buffer.toString();
    _append(io_textArea, stackTrace);
    io_textArea.insert("", 0); //scrolls back upwards to character first line
    _dialog.pack();
    _stackTraceShown = true;
  }

  //attributes:

  /**Dialog window for reporting the exception cause chain*/
  private final java.awt.Dialog _dialog;

  private boolean _stackTraceShown = false;

}//Dialog


/**Returns the owner window (which should be a Frame or Dialog),
  to wich i_component belongs to.
  Returns the nearest java.awt.Window, received by repeated invocation of
  java.awt.Component.getParent().
  Returns null, if no such java.awt.Window is found.
*/
private static java.awt.Window _getOwnerWindow(final java.awt.Component i_component){
  for(java.awt.Component c=i_component; c!=null; c=c.getParent()){
    if(c instanceof java.awt.Window){
      return (java.awt.Window)c;
    }
  }
  System.err.println(
    _className + ": NO OWNER Window FOUND FOR Component " + i_component
  );
  return null;
}//_getOwnerWindow

/**
 * Returns a package-private, shared, invisible Frame
 * to be the owner for Dialogs created with null owners.
 */
static java.awt.Frame getSharedOwnerFrame(){
  return _sharedOwnerFrame;
}

/**package-private, shared, invisible Frame*/
private static final java.awt.Frame _sharedOwnerFrame = new java.awt.Frame();

/**Reports to System.err, that exception i_problem occured when
  trying to report the exception i_reportee.
*/
private static void _reportNoWindow(
  final Throwable i_problem, final Throwable i_reportee
){
  System.err.println();
  System.err.print(_className);
  System.err.println(": THE FOLLOWING EXCEPTION OCCURED:");
  i_problem.printStackTrace();
  System.err.print(_className);
  System.err.println(": WHEN CONSTRUCTING THE MESSAGE WINDOW FOR EXCEPTION:");
  i_reportee.printStackTrace();
}

/**Appends i_text to io_textArea and enlarges the number of rows of this
  TextArea as necessary
*/
private static void _append(
  final java.awt.TextArea io_textArea, final String i_text
){
  io_textArea.setRows( io_textArea.getRows() + _countLines(i_text) );
  io_textArea.append(i_text);
}

/**Result: Number of lines in String i_text
  @see java.io.BufferedReader#readLine()
*/
private static int _countLines(final String i_text){
  final java.io.BufferedReader br
  = new java.io.BufferedReader(new java.io.StringReader(i_text));
  for(int result=0;;result++){
    String line; //final
    try{line = br.readLine();}
      catch(final java.io.IOException ex){line = null;}
    //end try
    if(line==null){return result;}
  }
}//_countLines

/**User should not construct objects of this class*/
private Awt(){}

    /**Prints i_component and the chain of its parents for test purposes.
    private static void _printParents(final java.awt.Component i_component){
      System.err.println("Component: ");
      for(java.awt.Component c=i_component;; c=c.getParent()){
        System.err.println(c);
        System.err.println("getParent(): ");
        if(c==null){break;}
      }
    }//_printParents
    */


}//Awt

/* Knabe 1999-03-18
Unerklaerlicherweise muss zuerst der traceButton, dann die textArea in den
Dialog eingefuegt werden. Bei der umgekehrten Reihenfolge tritt in der
folgenden Situation:
    nach Ausloesen der Ausname FigurNichtErzeugbar: Rechtspfeil
    und dann Druecken des Buttons <Print Stack Trace>
    und dann Klicken des <Fenster zu>-Kreuzes
die folgende, unmotivierte Ausnahme auf:
    Exception occurred during event dispatching:
    java.lang.NullPointerException: invalid peer
        at java.awt.Component.removeNotify(Component.java:2523)
        at java.awt.TextComponent.removeNotify(TextComponent.java:106)
        at java.awt.Container.removeNotify(Container.java:1147)
        at java.awt.Window.dispose(Window.java:177)
        at lib.Awt$1.windowClosing(Awt.java:63)
*/