package multex; //MsgText.java

import java.text.MessageFormat;
import java.util.Locale;



//Change history:
//2003-05-22  Knabe  Enables dynamic internationalization. See appendMessageChain.
//2001-02-26  Knabe  Internationalization of message texts is now optional.
//2000-10-25  Knabe  appendMessageChain and appendMessageLine append to a
//                   StringBuffer rather than returning a String.
//2000-07-07  Knabe  extrahiert aus Klasse Msg


/** Services for getting the message text for exceptions.
  It handles exceptions <UL>
    <LI>with parameters (subclasses of {@link Exc})
    <LI>with cause chain and parameters (subclasses of {@link Failure})
    <LI>and standard Java exceptions (other subclasses of java.lang.Throwable)
  </UL>
  For normal message reporting you should use the services of one of the
  destination oriented reporting classes, e.g. {@link Msg} or {@link Awt}.
  
  @author Christoph Knabe, Berlin, Copyright 1999-2003
*/
public class MsgText {


private static final String _className = new MsgText().getClass().getName();
public  static final String resourceBundleName = "MsgText";
public  static final String causeMarker = "CAUSE: ";

/**If true, Activates the static default internationalization of message texts. Further calls to
  {@link #appendMessageLine} or {@link #appendMessageChain} will deliver the
  localized message text for an exception instead of the default message
  text contained in the MultexException-object.
  The localized message text will be taken from the resource bundle
  with the name {@link #resourceBundleName}.
  
  If false, deactivates static internationalization. The Message text for a
  MultexException will be taken directly from the exception object itself.
  
  @since MulTEx 1c
*/
public static void setInternationalization(final boolean i_internationalization){
    _internationalization = i_internationalization;
    if(_internationalization){
    	_locale = new MultexLocale(_tryLoadMessages(), null, null);
    }else{
    	_locale = MultexLocale.nullLocale;
    }
}


/**Activates a specific static internationalization of message texts. 
 * Further calls to
  {@link #appendMessageLine} or {@link #appendMessageChain} will deliver the
  localized message text for an exception instead of the default message
  text contained in the MultexException-object.

  @param i_locale How to localize exception message texts
  
  @throws IllegalArgumentException If null is provided as i_locale

  @since MulTEx 4
  
  @see #setInternationalization(boolean)
*/
public static void setInternationalization(final MultexLocale i_locale)
throws IllegalArgumentException
{
	if(i_locale==null){throw new IllegalArgumentException("MultexLocale must not be null");}
    _internationalization = true;
	_locale = i_locale;
}

/**Returns the actual MultexLocale used for static or no internationalization.*/
public static MultexLocale getLocale(){return _locale;}

/**Appends the message for i_throwable with its causal chain.
  Appends an usually multi-line message, in which the first line is the
    message for i_throwable and the further lines are the messages in the
    chain of causing exceptions of i_throwable.
    See {@link #appendMessageLine} for a despription,
    how each exception is transformed into a message line.
  @param i_lineSeparator The String used to separate the lines of the message chain.
    If null is provided, defaults to {@link Failure#lineSeparator}.
  @deprecated since MulTEx 4 (2003-05-27). Use the more parameterized version instead.
*/
public static void appendMessageChain(
  final StringBuffer io_destination,
  final Throwable i_throwable,
  final String i_lineSeparator
){
  appendMessageChain(
    io_destination, i_throwable, _locale, i_lineSeparator
  );
}



/**Appends the message for i_throwable with its causal chain.
  Appends an usually multi-line message, in which the first line is the
    message for i_throwable and the further lines are the messages in the
    chain of causing exceptions of i_throwable.
    See {@link #appendMessageLine} for a despription,
    how each exception is transformed into a message line.

  @param io_destination   Where to append the message chain.
  @param i_throwable      The exception, which has to be reported along with its causal chain.
  @param i_locale         How the messsage text parameters are formatted.
                           If null,defaults to the default Locale.
  @param i_lineSeparator  The String used to separate the lines in the message chain.

  @since MulTEx 4 (2003-05-27) with dynamic internationalization.
*/
/*package*/ static void appendMessageChain(
  final StringBuffer             io_destination,
  final Throwable                 i_throwable,
  final MultexLocale              i_locale,
  final String                    i_lineSeparator
){
  final String causeSeparator
    = (i_lineSeparator!=null ? i_lineSeparator : Failure.lineSeparator)
    + i_locale.getCauseMarker()
  ;
  //Throwable-Meldungen eintragen:
  for(Throwable t=i_throwable; t!=null; t=Failure.getCause(t)){
    if(t!=i_throwable){ //causing exception
      io_destination.append(causeSeparator);
    }
    appendMessageLine(io_destination, t, i_locale);
  }
}


/**Appends the usually one-line message text for the exception
  i_throwable with inserted parameters to io_buffer.

  The message text pattern for the exception is taken from the mostly localized
  variant of the properties file "MsgText.properties" or helpwise from the
  default message text pattern in the exception object itself. A local message
  text pattern for the german language (de) in its swiss (CH) variant would be
  taken from file "MsgText_de_CH.properties", see the Java Tutorial
  <A HREF="http://java.sun.com/docs/books/tutorial/i18n/format/messageFormat.html">
  chapter on internationalization</A> at february 1999.
  <P>
  This file must contain definition lines of the form: <PRE>
     packageName.ExceptionName = message text pattern
  </PRE>or<PRE>
     packageName.ClassName$InnerExceptionName = message text pattern
  </PRE>with parameter substitutors in the message text pattern, e.g.:<PRE>
     CopyNP$CopyFailure = The file "{0}" could not be copied to "{1}"
  </PRE>
*/
public static void appendMessageLine(final StringBuffer io_destination, final Throwable i_throwable){
	appendMessageLine(io_destination, i_throwable, _locale);
}

/**Appends the usually one-line message text for the exception
 * i_throwable with inserted parameters to io_buffer, localizable.
 *
 * Note: You usually should pass only a ResourceBundle to i_messages, 
 * which was loaded with the same Locale, which you pass to i_locale.
 *
 * @param i_locale Source for the message text patterns instead of the default, 
 *                  the text pattern is taken with the class name of i_throwable
 *                  as key from i_locale.getMessages().
                    Controls the formatting of the message parameters, too.
 * @see #appendMessageLine(StringBuffer, Throwable) for details of parameter substitution
 * */
public static void appendMessageLine(
	final StringBuffer io_destination,
	final Throwable i_throwable, 
	final MultexLocale i_locale
) {
	  try {
	    final String textPattern = getMessageTextPattern(i_throwable, i_locale);
	    final Object[] parameters = _getParameters(i_throwable);
	    if(parameters==null){ //no insertable exception parameters
	      io_destination.append(textPattern);
	    }else{ //with exception parameters:
	      final MessageFormat format = new MessageFormat(textPattern);
	      format.setLocale(i_locale.getLocale());
	      format.format(parameters, io_destination, null);
	    }
	    if(!(i_throwable instanceof MultexException)){
	      //other Java exceptions may have detailMessage:
	      final String detailMessage = i_throwable.getMessage();
	      if(detailMessage!=null && detailMessage.length()!=0){
	        //Append standard Java detailMessage, separated by a colon
	        io_destination.append(": ");
	        io_destination.append(detailMessage);
	      }
	    }
	  } catch(final Exception ex){
	      System.err.println();
	      System.err.println(_className + ": EXCEPTION OCCURRED WHEN PREPARING MESSAGE LINE");
	      ex.printStackTrace();
	      io_destination.append(Failure.lineSeparator);
	      io_destination.append(i_throwable.toString());
	    }
	  //end try
}//appendMessageLine

/**Gets a message text pattern for the exception i_throwable.
  The following sources are tried one after another with the class name
  of i_throwable as message text key: <OL>
    <LI>The language and country specific MsgText properties file,
        e.g. MsgText_en_UK.properties
    <LI>The language specific MsgText properties file,
        e.g. MsgText_en.properties
    <LI>The default MsgText properties file: MsgText.properties
    <LI>The default message text pattern from i_throwable, if it is a
        MultexException
    <LI>The class name of the exception i_throwable
  </OL>
*/
/*package*/ static String getMessageTextPattern(final Throwable i_throwable){
	return getMessageTextPattern(i_throwable, _locale);
}


/**Gets a message text pattern for the exception i_throwable.
  The following sources are tried one after another with the class name
  of i_throwable as message text key: <OL>
    <LI>The ResourceBundle i_locale.getMessages()
    <LI>The default message text pattern from i_throwable, if it is a
        MultexException
    <LI>The class name of the exception i_throwable
  </OL>
*/
/*package*/ static String getMessageTextPattern(
	final Throwable i_throwable, final MultexLocale i_locale
) {
	  final String className = i_throwable.getClass().getName();
	  final String pattern0 = i_throwable instanceof MultexException ?
	    ((MultexException)i_throwable).getDefaultMessageTextPattern() : className
	  ;
	  final String defaultPattern = pattern0==null ? className : pattern0;
	  if(i_locale==null){
	    return defaultPattern;
	  }
	  final java.util.ResourceBundle messages = i_locale.getMessages();
	  if(messages==null){
	    return defaultPattern;
	  }
	  try{return messages.getString(className);}
	    catch(java.util.MissingResourceException ex){
	      return defaultPattern;
	    }
	  //try
}//_getMessageTextPattern

/**Returns the parameters contained in i_throwable or null*/
private static Object[] _getParameters(final Throwable i_throwable){
  if(!(i_throwable instanceof MultexException)){return null;}
  return ((MultexException)i_throwable).getParameters();
}

/*debug
private static void _println(final String i_name, final Object[] i_array){
  System.err.print(i_name + ": ");
  for(int i=0; i<i_array.length; i++){
    System.err.print(i_array[i] + "  ");
  }
  System.err.println();
}
*/


/**The maximum number of index parameters, as allowed by
  java.text.MessageFormat
*/
private static final int _maxNumberOfIndexParams = 10;


//Message text access:

private static boolean                  _internationalization = false;
private static MultexLocale              _locale = MultexLocale.nullLocale;
private static boolean                  _loadMessagesTried = false;

/**If _internationalization is on, tries once to load the ResourceBundle
  resourceBundleName as message text file and returns it.
  If the loading fails, this method will return null,
  and will do nothing on further invocations.
*/
private static synchronized java.util.ResourceBundle _tryLoadMessages(){
  if(_loadMessagesTried){return null;}
  try{
    _loadMessagesTried = true;
    return java.util.ResourceBundle.getBundle(resourceBundleName);
  } catch(final java.util.MissingResourceException ex){ //resource not found
      System.err.println(ex);
      System.err.println(_className + ": Unable to load exception message texts"
        + " from the ResourceBundle\n" + resourceBundleName
      );
      return null;
    } catch(final Exception ex){ //not loadable
      ex.printStackTrace();
      return null;
    }
  //end try
}//_tryLoadMessages


/**Do not create objects of me!*/
private MsgText(){}

}//MsgText
