package multex;

/**
 * Packages all locale specific adjustments for dynamic internationalization.
 * 
 * In systems like web servers each request has to be answered in the language
 * of the requester. For correctly reporting an exception with its causal chain
 * you have to provide the ResourceBundle for the exception message texts,
 * the Locale controlling the message parameter formatting, and the String
 * for marking each cause message in the chain.
 * @author Christoph Knabe 2003-05-27
 *
 */
public class MultexLocale {
	private final java.util.ResourceBundle _messages;
	private final java.util.Locale         _locale;
	private final String                   _causeMarker;
	

	/**
	 * Initializes the object with message text patterns, 
	 * a locale, and a cause marker. Passing null to a parameter
	 * will result in using it's default value.
	 * 
	 * @param i_messages If null, message text patterns are taken from the MultexException object itself.
	 * @param i_locale If null, defaults to the default Locale.
	 * @param i_causeMarker If null, defaults to {@link MsgText#causeMarker}.
	 * 
	 * 
	 * @see #getMessages
	 * @see #getLocale
	 * @see #getCauseMarker
	 */
	public MultexLocale(
		final java.util.ResourceBundle i_messages,
		final java.util.Locale         i_locale,
		final String                   i_causeMarker
	) {
		_messages    = i_messages;
		_locale      = i_locale     ==null ? java.util.Locale.getDefault() : i_locale;
		_causeMarker = i_causeMarker==null ? MsgText.causeMarker           : i_causeMarker;
	}

	/**
	 * Returns the message text patterns to be used for reporting each exceptions.
	 */
	public java.util.ResourceBundle getMessages() {
		return _messages;
	}

	/**
	 * Returns the Locale controlling the message parameter formatting by java.text.MessageFormat.
	 */
	public java.util.Locale getLocale() {
		return _locale;
	}

	/**
	 * Returns the String marking each cause message in an exception message chain.
	 */
	public String getCauseMarker() {
		return _causeMarker;
	}
	
	/**A predefined MultexLocale having no messages, 
	 * the default java.util.Locale and the default cause marker.
	 **/
	public static final MultexLocale nullLocale = new MultexLocale(null, null, null);

}
